@@ -61,7 +61,7 @@ except ImportError:
         return _convert(node_or_string)
 
 
-__version__ = '1.2'
+__version__ = '1.2.1'
 __all__ = ['Client', 'Model', 'Record', 'RecordList', 'Service',
            'format_exception', 'read_config']
 
@@ -107,16 +107,12 @@ Usage (main commands):
 
 STABLE_STATES = ('uninstallable', 'uninstalled', 'installed')
 DOMAIN_OPERATORS = frozenset('!|&')
-# Supported operators are
-#   =, !=, >, >=, <, <=, like, ilike, in,
-#   not like, not ilike, not in, child_of
-# Not supported operators are
-#  - redundant operators: '<>', '=like', '=ilike'
-#  - future operator(s) (6.1): '=?'
+# Supported operators are:
+#   =, !=, >, >=, <, <=, like, ilike, in, not like, not ilike, not in, child_of
+#   =like, =ilike (6.0), =? (6.0)
 _term_re = re.compile(
-    '([\w._]+)\s*'
-    '(=|!=|>|>=|<|<=|(?<= )(?:like|ilike|in|not like|not ilike|not in|child_of))'
-    '\s*(.*)')
+    '([\w._]+)\s*'  '(=(?:like|ilike|\?)|[<>!]=|[<>=]'
+    '|(?<= )(?:like|ilike|in|not like|not ilike|not in|child_of))'  '\s*(.*)')
 _fields_re = re.compile(r'(?:[^%]|^)%\(([^)]+)\)')
 
 # Published object methods
@@ -976,46 +972,46 @@ class RecordList(object):
         # Bypass the __setattr__ method
         self.__dict__.update({
             # 'client': res_model.client,
+            'id': _ids,
+            '_model_name': res_model._name,
             '_model': res_model,
             '_idnames': ids,
-            '_ids': _ids,
             '_context': context,
         })
 
     def __repr__(self):
-        if len(self._ids) > 16:
-            ids = 'length=%d' % len(self._ids)
+        if len(self.id) > 16:
+            ids = 'length=%d' % len(self.id)
         else:
-            ids = self._ids
-        return "<RecordList '%s,%s'>" % (self._model._name, ids)
+            ids = self.id
+        return "<RecordList '%s,%s'>" % (self._model_name, ids)
 
     def __dir__(self):
         return ['__getitem__', 'read', 'write', 'unlink',
-                '_context', '_ids', '_idnames', '_model']
+                'id', '_context', '_idnames', '_model', '_model_name']
 
     def __len__(self):
-        return len(self._ids)
+        return len(self.id)
 
     def read(self, fields=None, context=None):
         """Wrapper for :meth:`Record.read` method."""
         if context is None and self._context:
             context = self._context
 
-        if not self._ids:
-            return []
-
         client = self._model.client
-        values = client.read(self._model._name, self._ids,
-                             fields, order=True, context=context)
-
-        if values:
-            if isinstance(values[0], dict):
+        if self.id:
+            values = client.read(self._model_name, self.id,
+                                 fields, order=True, context=context)
+            if values and isinstance(values[0], dict):
                 browse_values = self._model._browse_values
                 return [browse_values(v) for v in values]
+        else:
+            values = []
 
-            if '%(' not in fields:
-                field = self._model._fields[fields]
+        if isinstance(fields, basestring):
+            field = self._model._fields.get(fields)
 
+            if field:
                 if field['type'] == 'many2one':
                     rel_model = client.model(field['relation'])
                     return RecordList(rel_model, values, context=context)
@@ -1035,22 +1031,27 @@ class RecordList(object):
         if attr.startswith('__'):
             errmsg = "'RecordList' object has no attribute %r" % attr
             raise AttributeError(errmsg)
-        model_name = self._model._name
-        execute = self._model.client.execute
+        if attr == '_ids':
+            # deprecated since 1.2.1
+            warnings.warn("Attribute 'RecordList._ids' is deprecated, "
+                          "use 'RecordList.id' instead.")
+            return self.id
+        model_name = self._model_name
+        execute = self._model._execute
 
         def wrapper(self, *params, **kwargs):
             """Wrapper for client.execute(%r, %r, [...], *params, **kwargs)."""
             if context:
                 kwargs.setdefault('context', context)
-            return execute(model_name, attr, self._ids, *params, **kwargs)
+            return execute(attr, self.id, *params, **kwargs)
         wrapper.__name__ = attr
         wrapper.__doc__ %= (model_name, attr)
         self.__dict__[attr] = mobj = wrapper.__get__(self, type(self))
         return mobj
 
     def __setattr__(self, attr, value):
-        if attr in self._model._keys:
-            msg = "attribute %r is read-only"
+        if attr in self._model._keys or attr == 'id':
+            msg = "attribute %r is read-only; use 'RecordList.write' instead."
         else:
             msg = "has no attribute %r"
         raise AttributeError("'RecordList' object %s" % msg % attr)
@@ -1075,11 +1076,10 @@ class Record(object):
             self.__dict__['_name'] = res_name
         # Bypass the __setattr__ method
         self.__dict__.update({
-            'client': res_model.client,
+            'id': res_id,
             '_model_name': res_model._name,
             '_model': res_model,
             '_context': context,
-            'id': res_id,
         })
 
     def __repr__(self):
@@ -1122,14 +1122,12 @@ class Record(object):
         """
         if context is None and self._context:
             context = self._context
-        rv = self.client.read(self._model_name, self.id,
-                              fields, context=context)
+        rv = self._model.read(self.id, fields, context=context)
         if isinstance(rv, dict):
             return self._update(rv)
-        elif '%(' in fields:
-            return rv
-        else:
+        elif isinstance(fields, basestring) and '%(' not in fields:
             return self._update({fields: rv})[fields]
+        return rv
 
     def perm_read(self, context=None):
         """Read the metadata of the :class:`Record`.
@@ -1137,16 +1135,14 @@ class Record(object):
         Return a dictionary of values.
         See :meth:`Client.perm_read` for details.
         """
-        rv = self.client.perm_read(self._model_name, [self.id],
-                                   context=context)
+        rv = self._model._execute('perm_read', [self.id], context=context)
         return rv and rv[0] or None
 
     def write(self, values, context=None):
         """Write the `values` in the :class:`Record`."""
         if context is None and self._context:
             context = self._context
-        rv = self.client.execute(self._model_name, 'write', [self.id],
-                                 values, context=context)
+        rv = self._model._execute('write', [self.id], values, context=context)
         self._clear_cache()
         return rv
 
@@ -1157,8 +1153,7 @@ class Record(object):
         """Delete the current :class:`Record` from the database."""
         if context is None and self._context:
             context = self._context
-        rv = self.client.execute(self._model_name, 'unlink', [self.id],
-                                 context=context)
+        rv = self._model._execute('unlink', [self.id], context=context)
         self._clear_cache()
         return rv
 
@@ -1170,13 +1165,12 @@ class Record(object):
         """
         if context is None and self._context:
             context = self._context
-        new_id = self.client.execute(self._model_name, 'copy', self.id,
-                                     default, context=context)
+        new_id = self._model._execute('copy', self.id, default, context=context)
         return Record(self._model, new_id)
 
     def __dir__(self):
-        return ['client', 'read', 'write', 'copy', 'unlink',
-                '_context', 'id', '_model', '_model_name',
+        return ['read', 'write', 'copy', 'unlink',
+                'id', '_context', '_model', '_model_name',
                 '_name', '_keys', '_fields'] + self._model._keys
 
     def __getattr__(self, attr):
@@ -1188,13 +1182,17 @@ class Record(object):
             return name
         if attr.startswith('__'):
             raise AttributeError("'Record' object has no attribute %r" % attr)
+        if attr == 'client':
+            # deprecated since 1.2.1
+            warnings.warn("Attribute 'Record.client' is deprecated, "
+                          "use 'Record._model.client' instead.")
+            return self._model.client
 
         def wrapper(self, *params, **kwargs):
             """Wrapper for client.execute(%r, %r, %d, *params, **kwargs)."""
             if context:
                 kwargs.setdefault('context', context)
-            res = self.client.execute(
-                self._model_name, attr, [self.id], *params, **kwargs)
+            res = self._model._execute(attr, [self.id], *params, **kwargs)
             if isinstance(res, list) and len(res) == 1:
                 return res[0]
             return res
